﻿using System;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;

namespace Amplitude.AI.Core.Brain.Generators.Economy.StarSystemManagement
{
    public class IncreaseApprovalOnSystem : DefaultContextBasedGenerator<ColonizedStarSystemSnapshot, ImproveResourceProduction, ImproveResourceProduction>
    {
        public override string LocalizedName
        {
            get
            {
                return "Watch system approval";
            }
        }

        public override string Description
        {
            get
            {
                return "The system may have approval issues, which means we need to adress the situation quickly.";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 5;
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            MajorEmpireSnapshot majorEmpireSnapshot = aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot != null && majorEmpireSnapshot.UseHonor)
            {
                return;
            }
            for (int i = 0; i < aiPlayer.ControlledEmpireSnapshot.ColonizedStarSystems.Count; i++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot = aiPlayer.ControlledEmpireSnapshot.ColonizedStarSystems[i];
                if (colonizedStarSystemSnapshot.State == ColonizedStarSystemSnapshot.StarSystemState.Colony && colonizedStarSystemSnapshot.Happiness <= IncreaseApprovalOnSystem.MaximumHappinessThreshold)
                {
                    base.ContinueDecision(aiPlayer, colonizedStarSystemSnapshot);
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, ColonizedStarSystemSnapshot context)
        {
            ImproveResourceProduction parent;
            if (aiPlayer.Blackboard.TryGetRunning(aiPlayer.ControlledEmpireSnapshot.GameEntityGuid, ResourceProductionValues.Approval, out parent))
            {
                base.ContinueDecision(aiPlayer, context, parent);
            }
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, ColonizedStarSystemSnapshot context, ImproveResourceProduction parent)
        {
            ImproveResourceProduction improveResourceProduction;
            if (!aiPlayer.Blackboard.TryGetRunning(context.GameEntityGuid, ResourceProductionValues.Approval, out improveResourceProduction))
            {
                improveResourceProduction = new ImproveResourceProduction(aiPlayer.Blackboard.GenerateGUID(), context, ResourceProductionValues.Approval);
                aiPlayer.Blackboard.Register<ImproveResourceProduction>(improveResourceProduction);
            }
            base.ContinueDecision(aiPlayer, base.GetGenerationData(context, parent, improveResourceProduction));
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<ColonizedStarSystemSnapshot, ImproveResourceProduction, ImproveResourceProduction> generationData)
        {
            HeuristicFloat result = new HeuristicFloat(0f, "");
            result.Add(IncreaseApprovalOnSystem.MaximumHappinessThreshold, "Maximum happiness percentage above which we won't consider improving happiness");
            result.Subtract(generationData.Context.Happiness, "The current happiness percentage");
            result.Divide(IncreaseApprovalOnSystem.MaximumHappinessThreshold, "Convert happiness percentage to ratio of maximum happiness threshold");
            result.Smoothstep(3);
            if (generationData.Context.RebellionValue > 0f)
            {
                result.Boost(0.7f, "This system is contributing to the empire-level rebellion");
                if (generationData.Context.RebellionProgressPerTurn > 0f)
                {
                    result.Boost(0.5f, "This system is tending towards system-level rebellion");
                }
            }
            return result;
        }

        public IncreaseApprovalOnSystem()
        {
        }

        static IncreaseApprovalOnSystem()
        {
        }

        private static readonly float MaximumHappinessThreshold = 85f;
    }
}
